import random

random.seed('icpccamp')

def rand(a,b):
  return random.randint(a,b)

MAX_VAL = int(1e9)

def print_case(n, S, ps, qs, shuffle=False):
  res = '%d %d %d %d\n' % (n, len(ps), len(qs), S)
  if shuffle:
    random.shuffle(ps)
    random.shuffle(qs)
  res += ''.join([str(x) + '\n' for x in ps])
  res += ''.join([str(x) + '\n' for x in qs])
  return res
    
def range_random(args):
  """
  Picks values between [low, high] randomly (uniform probability).
  """
  n, p, q, S = int(args['n']), int(args['p']), int(args['q']), int(args['S'])
  low, high = int(args['low']), int(args['high'])
  ps = [rand(low, high) for _ in range(p)]
  qs = [rand(low, high) for _ in range(q)]
  return print_case(n, S, ps, qs)

def all_possible(args):
  """
  For each random value v in list P, generates S - v to put into Q.
  The allowed S is S * multiS.
  """
  n, S = int(args['n']), int(args['S'])
  multiS = int(args['multiS'])
  ps, qs = [], []
  for i in range(n):
    v = rand(0, S)
    ps.append(v)
    qs.append(S - v)
  return print_case(n, S * multiS, ps, qs, False)
    
def stairs(args):
  """
  Generates values in pairs.
  Each pair is at distance D from the previous pair, plus a uniform random offset between [-var, var].
  Sets S to be the sum of the last pair.
  Half of the pairs are required to be matched.
  """
  pq, var, D = int(args['pq']), int(args['var']), int(args['D'])
  ps, qs = [], []
  v1, v2 = 0, 0
  while len(ps) < pq:
    v1 = v1 + D + rand(-var, var)
    v2 = v2 + D + rand(-var, var)
    v1, v2 = min(v1, MAX_VAL), min(v2, MAX_VAL)
    ps.append(v1)
    qs.append(v2)
    if len(ps) == pq // 2:
      S = v1 + v2
  return print_case(pq // 2, S, ps, qs, True)

def upperbound_breaker(args):
  """
  Half at the top of P are exactly matched with the bottom of Q.
  The remaining half are mid_val away.
  
  [P] 1...n/2                             S-n/2+1...S
                                          
  [Q] 0...n/2-1      mid_val...mid_val+n/2       
              
  """
  n, S = int(args['n']), int(args['S'])
  ps, qs = [], []
  for v in range(1,n//2+1):
    ps.append(v)
  for v in range(S-n//2+1, S+1):
    ps.append(v)
    qs.append(S - v)
  mid_val = int(1e7)
  for v in range(mid_val,mid_val + n//2):
    qs.append(v)
  return print_case(n, S, ps, qs, True)
  
def vector_breaker(args):
  """
  The bottom of P matches the top of Q.
  The middle chunks are used to waste the time of vector.erase().
  
  [P] n/2...n    10n+1...10n+n/2
                                          
  [Q]            10n+1...10n+n/2         S-n...S-n/2      
              
  """
  n, S = int(args['n']), int(args['S'])
  ps, qs = [], []
  for v in range(n//2+1, n+1):
    ps.append(v)
    qs.append(S - v)
  for v in range(10*n+1, 10*n+n//2+1):
    ps.append(v)
    qs.append(v)
  return print_case(n, S, ps, qs, True)

def same(args):
  pq = int(args['pq'])
  v = int(args['v'])
  return print_case(pq - 1, v * 2, [v] * pq, [v] * pq, False) 
  
def ranges(args):
  """
  P goes from p_start to p_end.
  Q goes from q_start to q_end.
  """
  n, S = int(args['n']), int(args['S'])
  p_start, p_end = int(args['p_start']), int(args['p_end'])
  q_start, q_end = int(args['q_start']), int(args['q_end'])
  ps = list(range(p_start, p_end + 1))
  qs = list(range(q_start, q_end + 1))
  return print_case(n, S, ps, qs, True)

def distinct(args):
  """
  Generates n distinct integers and split them into two lists.
  """
  n, p, q, S = int(args['n']), int(args['p']), int(args['q']), int(args['S'])
  low, high = int(args['low']), int(args['high'])
  modulo = int(args['modulo'])
  assert modulo >= 11
  base_vals = set()
  vals = set()
  while len(vals) < p + q:
    b = rand(low, high)
    if b in base_vals: continue
    base_vals.add(b)
    v = b * modulo + rand(modulo // 2 - 5, modulo // 2 + 5)
    if v in vals: raise Exception('each base_vals should have only one value')
    vals.add(v)
  vals = list(vals)
  random.shuffle(vals)
  return print_case(n, S, vals[:p], vals[p:], False)
