// Uses ESP to find cases against ESP cheese.
#include <iostream>
#include <cstdio>
#include <cmath>
#include <cstring>
#include <algorithm>
#include <string>
#include <vector>
#include <stack>
#include <queue>
#include <set>
#include <map>
#include <sstream>
#include <fstream>
#include <complex>
#include <ctime>
#include <cassert>
#include <functional>
#include <memory>
#include <stdexcept>
#include <array>
#include <thread>

using namespace std;

typedef long long ll;
typedef vector<int> VI;
typedef pair<int, int> PII;

#define REP(i,s,t) for(int i=(s);i<(t);i++)
#define FILL(x,v) memset(x,v,sizeof(x))

const int INF = (int)1E9;
#define MAXN 32

string exec(const char* cmd) {
	std::array<char, 128> buffer;
	std::string result;
	std::unique_ptr<FILE, decltype(&pclose)> pipe(popen(cmd, "r"), pclose);
	if (!pipe) {
			throw std::runtime_error("popen() failed!");
	}
	while (fgets(buffer.data(), buffer.size(), pipe.get()) != nullptr) {
			result += buffer.data();
	}
	return result;
}

int TRIALS = 10;
void evaluate(int tid, pair<int, VI> &p) {
	VI &v = p.second;
	string filename = "input_esp/" + to_string(tid);
	ofstream file(filename, ios::out);
	file << "30 " << v.size() << endl;
	REP(i,0,v.size()) file << v[i] << endl;
	file.close();
	int user, ac;
	string result = exec(("./ac < " + filename).c_str());
	stringstream ssac(result);
	ssac >> ac;
	if (ac == 30) {
		p.first = TRIALS;
		return; // assume the cheese will always find 30
	}
	int ok = 0;
	REP(tr,0,TRIALS) {
	  result = exec(("java Main < " + filename).c_str());
	  stringstream ss(result);
	  ss >> user;
	  if (ac == user) ok++;
	}
	if (ok <= TRIALS / 2) {
		cout << ok << "/" << TRIALS << "  ";
		REP(i,0,v.size()) cout << v[i] << " ";
		cout << endl;
	}
	p.first = ok;
}

double prob() {
	return (double)rand() / RAND_MAX;
}

int rand(int low, int high) {
	return rand() % (high - low + 1) + low;
}

const int MAX_M = 10;
const int LAMBDA = 32;
const int MU = 10;
const int MAX_THREADS = 4;
int main() {
	srand(time(NULL));

	// population.push_back(make_pair(-42, VI({5, 2, 3, 3, 8, 10, 6,  3, 6})));
	// population.push_back(make_pair(-42, VI({2, 2, 4, 5, 8,  8, 4, 11, 3, 3})));
	vector< pair<int, VI> > population, newPopulation;
	REP(i,0,5) {
		VI vs;
		REP(i,0,MAX_M) vs.push_back(rand(2, 8));
		population.push_back(make_pair(-42, vs));
	}
	while (1) {
		VI needEval;
		REP(i,0,population.size()) {
			if (population[i].first == -42) {
				needEval.push_back(i);
			}
		}
		REP(tids,0,needEval.size()) {
			vector<thread> ts;
			REP(tid,tids,tids+MAX_THREADS) {
				if (tid >= needEval.size()) break;
				thread t(evaluate, tid, ref(population[needEval[tid]]));
				ts.push_back(move(t));
			}
			for (auto &t: ts) t.join();
			tids += MAX_THREADS - 1;
		}

		sort(population.begin(), population.end());
		if (population.size() > MU) population.erase(population.begin() + MU, population.end());
		newPopulation.clear();
		REP(it,0,LAMBDA) {
			int i = rand(0, population.size() - 1);
			VI &curVs = population[i].second;
			if (curVs.size() < MAX_M) {
				if (prob() < 0.2) {
					VI newVs = curVs;
					newVs.push_back(rand(1, 15));
					newPopulation.push_back(make_pair(-42, newVs));
				}
			}
			if (curVs.size() > 5) {
				if (prob() < 0.2) {
					VI newVs = curVs;
					int j = rand(0, newVs.size() - 1);
					swap(newVs[j], newVs[newVs.size() - 1]);
					newVs.pop_back();
					newPopulation.push_back(make_pair(-42, newVs));
				}
			}
			VI newVs = curVs;
			int j = rand(0, newVs.size() - 1);
			newVs[j] = rand(1, 15);
			newPopulation.push_back(make_pair(-42, newVs));
		}
		population = newPopulation;
	}
	return 0;
}
