import random
import StringIO
from subprocess import call, Popen, PIPE, STDOUT

random.seed('tombraider')

MAX_N = 500

def run(b):
  testcase = StringIO.StringIO()
  testcase.write(print_case(b))
  p = Popen(['./sol'], stdout=PIPE, stdin=PIPE)
  output = p.communicate(input=testcase.getvalue())[0]
  output_str = output.decode()
  testcase.close()
  return output_str
call(['g++', '-o', 'sol', '../submissions/accepted/tombraider_bowen.cpp', '-O3'])

def rand(a,b):
  return random.randint(a,b)

def print_case(b):
  n, m = len(b), len(b[0])
  res = '%d %d\n' % (n, m)
  res += ''.join([''.join(row) + '\n' for row in b])
  return res
  
def create_board(n, m):
  return [['.'] * m for _ in range(n)] 
  
def random_board(args):
  n, m = int(args['n']), int(args['m'])
  b = create_board(n, m)

def rand_gargoyle():
  return 'V' if rand(0, 1) == 0 else 'H'

def rand_mirror():
  return '\\' if rand(0, 1) == 0 else '/'

def valid(b, x, y, allow='*'):
  n, m = len(b), len(b[0])
  return x >= 0 and x < n and y >= 0 and y < m and (allow == '*' or b[x][y] in allow)


def create_gridgroups(n, m, rows, cols, G, gargoyle_rate):
  """
  Randomly splits rows and columns into G groups.
  Then randomly add gargoyles at the row-col intersections in each group.
  Note that because of randomness the result may actually contain more than G groups.
  """
  g_rows = [[] for _ in range(G)]
  g_cols = [[] for _ in range(G)]
  random.shuffle(rows)
  random.shuffle(cols)
  for i in range(G): # ensures every group has 1 row/col
    g_rows[i].append(rows[i])
    g_cols[i].append(cols[i])
  for r in rows[G:]:
    g_rows[rand(0, G - 1)].append(r)
  for c in cols[G:]:
    g_cols[rand(0, G - 1)].append(c)
  
  b = create_board(n, m)
  for i in range(G):
    rows, cols = g_rows[i], g_cols[i]
    spots = [(i, j) for j in cols for i in rows]
    for i, j in spots:
      b[i][j] = rand_gargoyle() if random.random() < gargoyle_rate else '.'
  return b, g_rows, g_cols

def random_gridgroups(args):
  """
  Creates cases based on create_gridgroups.
  """
  n, m = int(args['n']), int(args['m'])
  G = int(args['G'])
  gargoyle_rate = float(args['gargoyle_rate'])
  b, _, _ = create_gridgroups(n, m, list(range(n)), list(range(m)), G, gargoyle_rate)
  return print_case(b)

def fixed_gridgroups(args):
  """
  Based on gridgroups using only the even index columns.
  Try to fix directions of each group by adding an obstacle in the nearby empty columns. 
  """
  n, m = int(args['n']), int(args['m'])
  assert(n % 2 == 0 and m % 2 == 0)
  G = int(args['G'])
  gargoyle_rate = float(args['gargoyle_rate'])
  fix_rate = float(args['fix_rate'])
  b, g_rows, g_cols = create_gridgroups(n, m, list(range(0,n,2)), list(range(0,m,2)), G, gargoyle_rate)
  for i in range(G):
    rows, cols = g_rows[i], g_cols[i]
    spots = [(i, j) for j in cols for i in rows]
    locations = []
    for i, j in spots:
      if b[i][j] != '.':
        locations.append((i, j))
    if len(locations) == 0:
      print ('[warning] empty locations. are gargoyles too sparse?')
      continue
    i, j = random.choice(locations)
    if rand(0, 1) == 1:
      b[i + 1][j] = '#'
    else:
      b[i][j + 1] = '#'
  return print_case(b)
  
def spiral(args):
  """
  Draws spirals.
  """
  n, m = int(args['n']), int(args['m'])
  directions = [[1, 0], [0, 1], [-1, 0], [0, -1]]
  x, y, d = 0, 0, 0
  b = create_board(n, m)
  filled = 0
  while filled < n * m:
    nx, ny = x + directions[d][0], y + directions[d][1]
    if not valid(b, nx, ny, '.'):
      b[x][y] = '\\' if d % 2 == 0 else '/'
      d = (d + 1) % 4
      x, y = x + directions[d][0], y + directions[d][1]
    else:
      b[x][y] = rand_gargoyle()
      x, y = nx, ny
    filled += 1
  # create a life lane from the last location
  while valid(b, x, y, '*'):
    b[x][y] = '.'
    x, y = x + directions[d][0], y + directions[d][1]
  return print_case(b)

def random_case(args):
  """
  Generates board based on pure randomness.
  Must tune the ratio carefully to make the case useful.
  Record the seed, so that the case can be regenerated.
  """
  n, m = int(args['n']), int(args['m'])
  disallow_no = int(args['disallow_no'])
  mirror_rate = float(args['mirror_rate'])
  gargoyle_rate = float(args['gargoyle_rate'])
  obstacle_rate = float(args['obstacle_rate'])
  seed = int(args['seed'])
  gargoyle_rate += mirror_rate
  obstacle_rate += gargoyle_rate
  assert(obstacle_rate <= 1.0)
  
  while True:
    random.seed('tombraider%d' % seed)
    b = create_board(n, m)
    cnt_mirror, cnt_gargoyle, cnt_obstacle = 0, 0, 0
    for i in range(n):
      for j in range(m):
        t = random.random()
        if t < mirror_rate:
          b[i][j] = rand_mirror()
          cnt_mirror += 1
        elif t < gargoyle_rate:
          b[i][j] = rand_gargoyle()
          cnt_gargoyle += 1
        elif t < obstacle_rate:
          b[i][j] = '#'
          cnt_obstacle += 1
    ans = int(run(b))
    print (seed, ans, 'mirror', cnt_mirror, 'gargoyle', cnt_gargoyle, 'obstacle', cnt_obstacle)
    if disallow_no == 1 and ans == -1:
      seed += 1
      continue
    print ('use seed', seed, 'ans', ans)
    break
  return print_case(b)

def squares(args):
  """
  Every corner of squares are mirrors, like this
  /...\
  ./.\.
  .....
  .\./.
  \.../
  The empty spots are randomly filled with gargoyles.
  """
  n = int(args['n'])
  gargoyle_rate = float(args['gargoyle_rate'])
  b = create_board(n, n)
  for i in range(n):
    b[i][i] = b[n - 1 - i][n - 1 - i] = '/'
    b[i][n - 1 - i] = b[n - 1 - i][i] = '\\'
  for i in range(n):
    for j in range(n):
      if b[i][j] == '.' and random.random() < gargoyle_rate:
        b[i][j] = rand_gargoyle()
  return print_case(b)

def diagonals(args):
  """
  Create diagonal mirror channels.
  Randomly seal position 1 or 2 to control some directions.
  ..12.
  .V\H\
  .\.\.
  .H\.\
  """
  n, m = int(args['n']), int(args['m'])
  obstacle_rate = float(args['obstacle_rate'])
  gargoyle_rate = float(args['gargoyle_rate'])
  assert (n % 2 == 1 and m % 2 == 1)
  b = create_board(n, m)
  for i in range(1,n-1):
    for j in range(1,m-1):
      if (i + j) % 2 == 1:
        b[i][j] = '\\'
      elif i == 1 or j == 1 or i == n - 2 or j == m - 2 or random.random() < gargoyle_rate:
        b[i][j] = rand_gargoyle()
  for i in range(2,m,2):
    t = random.random()
    if t < obstacle_rate / 2:
      b[0][i] = '#'
    elif t < obstacle_rate:
      if i + 1 < m: b[0][i + 1] = '#'
  return print_case(b)

def squares_x(args):
  """
  Every corner of squares are mirrors (similar to squares), and we place gargoyles
  near the mirrors (X), like this:
  /X.....X\
  X/X...X\X
  .X/X.X\X.
  (and so on)
  """
  n = int(args['n'])
  gargoyle_rate = float(args['gargoyle_rate'])
  b = create_board(n, n)
  for i in range(n):
    b[i][i] = '/'
    if i + 1 < n and b[i + 1][i] == '.' and random.random() < gargoyle_rate:
      b[i + 1][i] = rand_gargoyle()
    if i + 1 < n and b[i][i + 1] == '.' and random.random() < gargoyle_rate:
      b[i][i + 1] = rand_gargoyle()
    
    b[i][n - 1 - i] = '\\'
    if i + 1 < n and b[i + 1][n - 1 - i] == '.' and random.random() < gargoyle_rate:
      b[i + 1][n - 1 - i] = rand_gargoyle()
    if n - 2 - i >= 0 and b[i][n - 2 - i] == '.' and random.random() < gargoyle_rate:
      b[i][n - 2 - i] = rand_gargoyle()
  return print_case(b)

def four_blocks(args):
  """
  X are mirrors. Split the area into 4 blocks.
  XX\\
  XX\\
  //XX
  //XX
  """
  n = int(args['n'])
  assert n % 2 == 0
  n2 = n // 2
  b = create_board(n, n)
  for i in range(n2):
    for j in range(n2):
      b[i][j] = rand_gargoyle()
      b[i + n2][j] = '/'
      b[i][j + n2] = '\\'
      b[i + n2][j + n2] = rand_gargoyle()
  return print_case(b)

def nine_blocks(args):
  """
  X are mirrors. Split the area into 9 blocks.
  XX..\\
  XX..\\
  //XX//
  //XX//
  XX..\\
  XX..\\
  """
  n = int(args['n'])
  assert n % 3 == 0
  n3 = n // 3
  b = create_board(n, n)
  for i in range(n3):
    for j in range(n3):
      b[i][j] = rand_gargoyle()
      b[i + n3][j + n3] = rand_gargoyle()
      b[i + n3 * 2][j] = rand_gargoyle()
      b[i + n3][j] = b[i + n3][j + n3 * 2] = '/'
      b[i][j + n3 * 2] = b[i + n3 * 2][j + n3 * 2] = '\\'
  return print_case(b)
  
def chessboard(args):
  n, step = int(args['n']), int(args['step'])
  assert n % step == 0
  b = create_board(n, n)
  for i in range(n // step):
    for j in range(n // step):
      mir = rand_mirror()
      is_gargoyle = (i + j) % 2 == 0
      for p in range(step):
        for q in range(step):
          b[i * step + p][j * step + q] = rand_gargoyle() if is_gargoyle else mir
  return print_case(b)

def long_odd_cycle(args):
  n, step = int(args['n']), int(args['step'])
  bad = int(args['bad'])
  assert n % step == 0
  assert step >= 2
  b = create_board(n, n)
  for i in range(n // step):
    for p in range(step):
      for q in range(step):
        b[i * step + p][i * step + q] = rand_gargoyle()
    if i < n // step - 1:
      b[i * step][n - 1] = '\\'
      b[i * step + step - 1][n - 1] = '/'
      b[i * step + step - 1][i * step + step] = '/'
  cnt = n // step
  if bad != cnt % 2:
    cnt -= 1
  b[step * cnt - 1][0] = '\\'
  return print_case(b)

def long_light(args):
  n = int(args['n'])
  bad = int(args['bad'])
  b = create_board(n, n)
  assert n % 2 == 0
  for i in range(n):
    for j in range(2,n):
      b[i][j] = '/' if i % 2 == 0 else '\\'
  for j in range(2,n):
    b[0][j] = b[n - 1][j] = '\\' if j % 2 == 0 else '/'
  b[0][2] = b[n - 1][n - 1] = 'H'
  b[0][0] = '#'
  for i in range(n):
    b[i][1] = rand_gargoyle()
  if bad:
    b[n - 2][n - 1] = '#'
  return print_case(b)

def long_light_loop(args):
  """
  Creates a long mirror loop without any gargoyle.
  """
  n = int(args['n'])
  b = create_board(n, n)
  assert n % 2 == 0
  for i in range(n - 1):
    for j in range(2,n):
      b[i][j] = '/' if i % 2 == 0 else '\\'
  for j in range(2,n):
    b[0][j] = '\\' if j % 2 == 0 else '/'
    b[n - 2][j] = '\\' if j % 2 == 1 else '/'
  b[0][1] = '/'
  b[n - 1][1] = '\\'
  b[n - 2][n - 1] = b[n - 1][n - 2] = '/'
  for i in range(n):
    b[i][0] = rand_gargoyle()
  return print_case(b)

def long_selfloop(args):
  """
  When n is odd, answer is -1.
  When n is even, answer is 0.
  """
  n = int(args['n'])
  x, y = n // 2, n // 2
  dirs = [[0, 1], [1, 0], [0, -1], [-1, 0]]
  d = 0
  b = create_board(n, n)
  b[x][y] = 'H'
  for s in range(1, n):
    x, y = x + dirs[d][0] * s, y + dirs[d][1] * s
    d = (d + 1) % 4
    b[x][y] = '\\'
    if n % 2 == 0 and x == n - 1:
      b[n // 2][y] = '/'
      break
    x, y = x + dirs[d][0] * s, y + dirs[d][1] * s
    d = (d + 1) % 4
    b[x][y] = '/'
    
    if x == 0:
      b[0][n // 2] = '\\'
      break
  return print_case(b)

def grid_intersect(args):
  n = int(args['n'])
  assert n % 2 == 0
  b = create_board(n, n)
  for i in range(n):
    b[i][0] = b[i][n - 1] = '/' if i % 2 == 1 else '\\'
  for j in range(1,n):
    b[0][j] = b[n - 1][j] = '/' if j % 2 == 1 else '\\'
  b[0][n - 1] = '#'
  b[2][0] = b[n - 1][0] = 'V'
  return print_case(b)

def grid_intersect2(args):
  n = int(args['n'])
  assert n % 2 == 0
  b = create_board(n, n)
  for i in range(1,n-1):
    b[i][0] = '/' if i % 2 == 1 else '\\'
    b[i][n - 1] = '/' if i % 2 == 0 else '\\'
  for j in range(1,n-1):
    b[0][j] = '\\' if j % 2 == 0 else '/'
    b[n - 1][j] = '\\' if j % 2 == 1 else '/'
  perm = list(range(1,n-1))
  random.shuffle(perm)
  row = 1
  for p in perm:
    b[row][p] = rand_gargoyle()
    row += 1
  return print_case(b)

def grid_intersect3(args):
  n = int(args['n'])
  maxlen = int(args['maxlen'])
  assert n % 2 == 0
  assert maxlen <= n - 2
  b = create_board(n, n)
  i = 1
  while i < n - 1:
    remlen = n - 2 - i + 1
    leng = rand(1, min(remlen, maxlen) // 2) * 2
    for k in range(i, i + leng):
      b[0][k] = b[n - 1][k] = '/' if (k - i) % 2 == 0 else '\\'
    b[n - 1][i] = '\\'
    b[n - 1][i + leng - 1] = '/'
    i += leng
  
  j = 1
  while j < n - 1:
    remlen = n - 2 - j + 1
    leng = rand(1, remlen // 2) * 2
    for k in range(j, j + leng):
      b[k][0] = b[k][n - 1] = '/' if (k - j) % 2 == 0 else '\\'
    b[j][n - 1] = '\\'
    b[j + leng - 1][n - 1] = '/'
    j += leng
 
  perm = list(range(1,n-1))
  random.shuffle(perm)
  row = 1
  for p in perm:
    b[row][p] = rand_gargoyle()
    row += 1
  return print_case(b)
  
def diamond(args):
  n, step = int(args['n']), int(args['step'])
  g = step * 2
  assert n % g == 0
  b = create_board(n, n)
  for i in range(n // g):
    for j in range(n // g):
      for k in range(step):
        b[i * g + k][j * g + step - 1 - k] = b[i * g + k + step][j * g + g - 1 - k] = '/'
        b[i * g + k][j * g + step + k] = b[i * g + k + step][j * g + k] = '\\'
  for i in range(n):
    for j in range(n):
      if b[i][j] == '.':
        b[i][j] = rand_gargoyle()
  return print_case(b)
